@@ -231,7 +231,10 @@ cd ../
 
 
 if [ ! -e "$DIRECTORY.orig.tar.gz" ]; then
-  tar zcf $DIRECTORY.orig.tar.gz $DIRECTORY.orig
+  read -p "$DIRECTORY.orig.tar.gz does not exist, create it? [Y/n]"
+  if [[ "$REPLY" == "" || "$REPLY" == [yY] ]]; then
+    tar zcf $DIRECTORY.orig.tar.gz $DIRECTORY.orig
+  fi;
 fi;
 
 IFS=',' ;for DISTRO in `echo "$DISTROS"`; do 
@@ -361,7 +364,7 @@ EOF
     dput="Y";
     if [ "$INTERACTIVE" != "no" ]; then
       read -p "Ready to dput $SC to $PPA ? Y/n...";
-      if [[ "$REPLY" == [yY] ]]; then
+      if [[ "$REPLY" == "" || "$REPLY" == [yY] ]]; then
         dput $PPA $SC
       fi;
     else
